package com.glintt.at.dmis.ws.core;

import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Set;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

/**
 * @author Ricardo Rodrigues (created based on the applet source code available on
 *         https://faturas.portaldasfinancas.gov.pt/testarLigacaoWebService.action )
 */
public class LoggingHandler implements SOAPHandler<SOAPMessageContext> {

    private final Logger logger;

    public LoggingHandler(final Logger logger) {
        this.logger = logger;
    }

    public Set<QName> getHeaders() {
        return null;
    }

    public boolean handleMessage(final SOAPMessageContext messageContext) {
        this.interceptAndRecordSoapMessageContext(messageContext);
        return true;
    }

    public boolean handleFault(final SOAPMessageContext messageContext) {
        this.interceptAndRecordSoapMessageContext(messageContext);
        return true;
    }

    // nothing to clean up
    public void close(final MessageContext messageContext) {
    }

    private void interceptAndRecordSoapMessageContext(final SOAPMessageContext messageContext) {

        final Logger messageLogger = this.logger;
        final SOAPMessage message = messageContext.getMessage();

        final boolean isRequest = (Boolean) messageContext.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

        try {
            final SOAPPart soapPart = message.getSOAPPart();
            final Source source = soapPart.getContent();
            messageLogger.info((Boolean) messageContext.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY)
                    ? "REQ" + this.sourceToXMLString(source) : this.sourceToXMLString(source));
        } catch (final Exception e) {
            final StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw));
            final String stacktrace = sw.toString();
            this.logger.severe("Could not intercept and log soap message\n\n" + stacktrace);
        }
    }

    private String sourceToXMLString(final Source result) throws TransformerConfigurationException,
            TransformerException, UnsupportedEncodingException {

        String xmlResult = null;
        try {
            final Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "8");
            transformer.setOutputProperty(OutputKeys.METHOD, "xml");
            transformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
            final ByteArrayOutputStream out = new ByteArrayOutputStream();
            transformer.transform(result, new StreamResult(out));
            xmlResult = out.toString("UTF-8");
        } catch (final TransformerException e) {
            final StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw));
            final String stacktrace = sw.toString();
            this.logger.severe("Could not intercept and log soap message\n\n" + stacktrace);
        }
        return xmlResult;
    }
}
