package com.glintt.at.dmis.ws.gui;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.logging.Logger;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.glintt.at.dmis.ws.TestClientApp;
import com.glintt.at.dmis.ws.proxy.DmisClientProxy;

import jsyntaxpane.DefaultSyntaxKit;

/**
 * @author Ricardo Rodrigues (created based on the applet source code available on
 *         https://faturas.portaldasfinancas.gov.pt/testarLigacaoWebService.action )
 */
public final class TestClientGuiFactory {

    

    private static final String DATAPOWER_QUA_HTTPS_INTERNET_ENDPOINT = "https://servicos.portaldasfinancas.gov.pt:721/DmisServiceImplService";
    
    private static final String[] ENDPOINTS = {
    TestClientGuiFactory.DATAPOWER_QUA_HTTPS_INTERNET_ENDPOINT };

    private static final String DEFAULT_NIFUSER = "599999993/0235";
    private static final String DEFAULT_PASSWORD = "testes1234";

    private static Cursor waitCursor = new Cursor(Cursor.WAIT_CURSOR);
    private static Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    private TestClientGuiFactory() {
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    public static TestClientPanel createTestClientGuiPanel() {

        final TestClientPanel panel = new TestClientPanel();

        panel.getUserTf().setText(TestClientGuiFactory.DEFAULT_NIFUSER);
        panel.getPasswordTf().setText(TestClientGuiFactory.DEFAULT_PASSWORD);

        panel.getEndpointCb().setModel(new DefaultComboBoxModel(TestClientGuiFactory.ENDPOINTS));

        final JEditorPane editorPane = panel.getSoapBodyEditorPane();

        DefaultSyntaxKit.initKit();
        editorPane.setContentType("text/xml");

        panel.getRequestLogTextArea().setLineWrap(true);
        panel.getRequestLogTextArea().setWrapStyleWord(true);
        panel.getResponseLogTextArea().setLineWrap(true);
        panel.getResponseLogTextArea().setWrapStyleWord(true);
        TestClientGuiFactory.addActionListeners(panel);
        panel.setVisible(true);

        return panel;
    }

    private static void addActionListeners(final TestClientPanel panel) {

        // log handler para escrita na textarea da UI
        final Logger logger = Logger.getLogger(TestClientApp.class.getSimpleName());
        logger.setUseParentHandlers(false);
        logger.addHandler(new JTextAreaLoggerHandler(panel.getRequestLogTextArea(),
                panel.getResponseLogTextArea()));

        panel.getTestBt().addActionListener(new ActionListener() {

            public void actionPerformed(final ActionEvent e) {
                panel.getRequestLogTextArea().setText("");
                panel.getResponseLogTextArea().setText("");

                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        try {
                            final String endpoint = TestClientGuiFactory.endPointConvert(
                                    String.valueOf(panel.getEndpointCb().getSelectedItem()));
                            final String user = panel.getUserTf().getText();
                            @SuppressWarnings("deprecation")
                            final String pass = panel.getPasswordTf().getText();
                            final String soapBody = panel.getSoapBodyEditorPane().getText();

                            if (endpoint == null || endpoint.equals("")) {
                                JOptionPane.showMessageDialog(panel, "Endereo Invlido", "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            }
                            if (user == null || user.equals("")) {
                                JOptionPane.showMessageDialog(panel, "NIF Invlido", "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            }

                            if (pass == null || pass.equals("")) {
                                JOptionPane.showMessageDialog(panel, "Password Invlida", "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            }

                            if (soapBody == null || soapBody.equals("")) {
                                JOptionPane.showMessageDialog(panel, "Request Body Invlido", "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            }

                            Document xmlBody = null;
                            try {
                                final DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
                                        .newInstance();
                                docBuilderFactory.setNamespaceAware(true);
                                final DocumentBuilder docBuilder = docBuilderFactory
                                        .newDocumentBuilder();
                                xmlBody = docBuilder.parse(new InputSource(new StringReader(soapBody)));
                            } catch (final ParserConfigurationException ex1) {
                                JOptionPane.showMessageDialog(panel,
                                        "Request Body XML Invlido: " + ex1.getMessage(), "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            } catch (final SAXException ex2) {
                                JOptionPane.showMessageDialog(panel,
                                        "Request Body XML Invlido: " + ex2.getMessage(), "Error",
                                        JOptionPane.ERROR_MESSAGE);
                                return;
                            }
                            panel.getParent().setCursor(TestClientGuiFactory.waitCursor);

                            final DmisClientProxy taxFreeClientProxy = new DmisClientProxy(logger,
                                    endpoint, user, pass);
                            taxFreeClientProxy.invokeWs(xmlBody);

                        } catch (final Exception ex) {
                            panel.getParent().setCursor(TestClientGuiFactory.defaultCursor);

                            final StringWriter sw = new StringWriter();
                            ex.printStackTrace(new PrintWriter(sw));
                            final String stacktrace = sw.toString();
                            logger.severe("***StackTrace***\n" + stacktrace);
                            JOptionPane.showMessageDialog(panel,
                                    ex.getClass().getSimpleName() + "\n\n" + ex.getMessage(), "Erro",
                                    JOptionPane.ERROR_MESSAGE);
                        }

                        panel.getParent().setCursor(TestClientGuiFactory.defaultCursor);
                    }
                });
            }
        });

    }

    private static String endPointConvert(final String displayed) {

        return displayed.contains("-") ? displayed.substring(displayed.indexOf("-") + 2) : displayed;
    }
}
