package com.glintt.at.dmis.ws.proxy;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.List;
import java.util.logging.Logger;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;

import org.w3c.dom.Document;

import com.glintt.at.dmis.ws.client.DmisService;
import com.glintt.at.dmis.ws.client.DmisServiceImplService;
import com.glintt.at.dmis.ws.core.LoggingHandler;
import com.glintt.at.dmis.ws.core.SoapBodyHandler;
import com.glintt.at.dmis.ws.core.WSSHeaderHandler;
import com.sun.xml.ws.developer.JAXWSProperties;
import com.sun.xml.ws.developer.WSBindingProvider;

/**
 * @author Ricardo Rodrigues (created based on the applet source code available on
 *         https://faturas.portaldasfinancas.gov.pt/testarLigacaoWebService.action )
 */
public class DmisClientProxy {

    private static final int DEFAULT_CONNECT_TIMEOUT = 1000;
    private static final int DEFAULT_REQUEST_TIMEOUT = 1000;

    private static final String BASIC_AUTH_USERNAME = "dmisws";
    private static final String BASIC_AUTH_PASSWORD = "12345678";

    private static final String WSDL_URL = "dmisWSDL/dmisSoapWebServices.wsdl";

    private static final String KEYSTORE_FILE_TESTE = "TesteWebServices.pfx";
    private static final String KEYSTORE_PASSWORD_TESTE = "TESTEwebservice";

    private static final String KEYSTORE_FILE = "599999993.pfx";
    private static final String KEYSTORE_PASSWORD = "x";

    private final Logger logger;
    private final String endpoint;
    private final String user;
    private final String password;

    public DmisClientProxy(final Logger logger, final String endpoint, final String user,
            final String password) {
        this.logger = logger;
        this.endpoint = endpoint;
        this.user = user;
        this.password = password;
    }

    public final void invokeWs(final Document xmlBody)
            throws UnrecoverableKeyException, KeyManagementException, KeyStoreException,
            NoSuchAlgorithmException, CertificateException, IOException {

        final URL wsdlUrl = this.getClass().getClassLoader().getResource(DmisClientProxy.WSDL_URL);

        final DmisServiceImplService service = new DmisServiceImplService(wsdlUrl);

        final DmisService servicePort = service.getDmisServicePort();

        @SuppressWarnings("resource")
        final WSBindingProvider wsBindingProvider = (WSBindingProvider) servicePort;

        wsBindingProvider.getRequestContext().put("javax.xml.ws.service.endpoint.address",
                this.endpoint);
        wsBindingProvider.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout",
                DmisClientProxy.DEFAULT_CONNECT_TIMEOUT);
        wsBindingProvider.getRequestContext().put("com.sun.xml.internal.ws.request.timeout",
                DmisClientProxy.DEFAULT_REQUEST_TIMEOUT);
        wsBindingProvider.getRequestContext().put(BindingProvider.USERNAME_PROPERTY,
                DmisClientProxy.BASIC_AUTH_USERNAME);
        wsBindingProvider.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY,
                DmisClientProxy.BASIC_AUTH_PASSWORD);

        // caso o endpoint seja https, adiciona o sslContext
        if (this.endpoint.startsWith("https")) {
            if (this.endpoint.contains("421")) {
                wsBindingProvider.getRequestContext().put(JAXWSProperties.SSL_SOCKET_FACTORY,
                        this.getSslContext(false).getSocketFactory());
            } else {
                wsBindingProvider.getRequestContext().put(JAXWSProperties.SSL_SOCKET_FACTORY,
                        this.getSslContext(true).getSocketFactory());
            }
        }

        @SuppressWarnings("rawtypes")
        final List<Handler> handlersChain = wsBindingProvider.getBinding().getHandlerChain();
        handlersChain.add(new WSSHeaderHandler(this.logger, this.user, this.password));
        handlersChain.add(new SoapBodyHandler(this.logger, xmlBody));
        handlersChain.add(new LoggingHandler(this.logger));

        wsBindingProvider.getBinding().setHandlerChain(handlersChain);

        final String requestElement = xmlBody.getFirstChild().getLocalName();
        final int requestIndex = requestElement.indexOf("Request");

        if (requestIndex < 0) {
            throw new RuntimeException("Soap Body XML com estrutura desconhecida: " + requestElement);
        }

        final String request = requestElement.substring(0, requestIndex);

        if (request.equals("DmisWsSubmission")) {
            servicePort.dmisSubmission(null);
        } else {
            throw new RuntimeException("Soap Body XML com estrutura desconhecida: " + requestElement);
        }
    }

    private SSLContext getSslContext(final boolean isTeste)
            throws KeyStoreException, IOException, NoSuchAlgorithmException, CertificateException,
            UnrecoverableKeyException, KeyManagementException {

        final InputStream keyStoreInputStream = this.getClass().getClassLoader().getResourceAsStream(
                isTeste ? DmisClientProxy.KEYSTORE_FILE_TESTE : DmisClientProxy.KEYSTORE_FILE);

        final KeyStore keyStore = KeyStore.getInstance("pkcs12");
        keyStore.load(keyStoreInputStream,
                isTeste ? DmisClientProxy.KEYSTORE_PASSWORD_TESTE.toCharArray()
                        : DmisClientProxy.KEYSTORE_PASSWORD.toCharArray());

        final KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
        kmf.init(keyStore, isTeste ? DmisClientProxy.KEYSTORE_PASSWORD_TESTE.toCharArray()
                : DmisClientProxy.KEYSTORE_PASSWORD.toCharArray());

        final SSLContext sslContext = SSLContext.getInstance("TLS");
        sslContext.init(kmf.getKeyManagers(), null, null);

        return sslContext;
    }
}